;;; $Id: write-gsc.scm,v 1.1 2021/12/20 00:00:22 lucier Exp lucier $ 
;;; Copyright (c) 2015 by Bradley Lucier. All rights reserved.

;;;                ***********************************************
;;;                    Small, Efficient and Innovative Class System
;;;                                    Meroon 
;;;                              Christian Queinnec  
;;;                    Ecole Polytechnique & INRIA--Rocquencourt
;;;                ************************************************

;;; A hook into Gambit's write system to display Meroon objects

(include "~~/lib/_gambit#.scm")

(##main-readtable-set! (readtable-sharing-allowed?-set ##main-readtable #t))

(##wr-meroon-set!
 (lambda (we obj)

   (define wr-field
     (lambda (we field)
       ;; We abuse ##wr-vector-aux1 from _io.scm to write fields with the correct spacing, etc.
       (if (Mono-Field? field)
           (##wr-vector-aux1 we
                             (##vector (Field-name field) (if (field-defined? obj field)
                                                              (field-value obj field)
                                                              '<meroon-uninitialized>))
                             2
                             ##vector-ref
                             '("(" . ")"))
           (let ((n (field-length obj field)))
             (if (##real? n)
                 (do ((i 0 (##fx+ i 1))
                      (result '() (##cons (if (field-defined? obj field i)
                                              (field-value obj field i)
                                              '<meroon-uninitialized>)
                                          result)))
                     ((##fx= i n)
                      (##wr-vector-aux1 we
                                        (##apply ##vector (##cons (Field-name field) (##reverse result)))
                                        (##fx+ n 1)
                                        ##vector-ref
                                        '("(" . ")"))))
                 (##wr-vector-aux1 we
                                   (##vector (Field-name field) '<meroon-uninitialized>)
                                   2
                                   ##vector-ref
                                   '("(" . ")")))))))
   

   (define (wr-meroon we obj)

     ;; We steal from ##wr-structure in _io.scm

     (##wr-str we "#m<")

     (let ((level
            (macro-writeenv-level we))
           (class
            (object->class obj)))
       (##wr-no-display we (Class-name class))
       (##wr-ch we #\space)
       
       (if (##not (##fx< level
                         (macro-readtable-max-write-level
                          (macro-writeenv-readtable we))))
           (##wr-str we "...")
           (let* ((close-parens
                   (macro-writeenv-close-parens we))
                  (new-close-parens
                   (##fx+ close-parens 1))
                  (fields
                   (Class-fields class)))
             (macro-writeenv-level-set! we (##fx+ level 1))
             (let ((start-col
                    (##shifted-column we)))
               (let loop ((i 0)
                          (fields fields))
                 (if (##fx< 0 (macro-writeenv-limit we))
                     (if (##not (##null? fields))
                         (let ()
                           (if (##fx< 0 i)
                               (case (macro-writeenv-style we)
                                 ((pretty-print) (##wr-indent we start-col))
                                 (else           (##wr-ch we #\space))))
                           (if (##not (##fx< i
                                             (macro-readtable-max-write-length
                                              (macro-writeenv-readtable we))))
                               (##wr-str we "...")
                               (let ((elem
                                      (##car fields))
                                     (new-i
                                      (##fx+ i 1)))
                                 (macro-writeenv-close-parens-set!
                                  we
                                  (if (##null? (##cdr fields))
                                      new-close-parens
                                      0))
                                 (wr-field we elem)
                                 (loop new-i
                                       (cdr fields)))))))))
             (macro-writeenv-level-set! we level)
             (macro-writeenv-close-parens-set! we close-parens))))

     (##wr-str we ">"))

   (case (macro-writeenv-style we)
     ((mark)
      (if (##wr-mark we obj)
          (##wr-vector-aux2 we obj (##vector-length obj) ##vector-ref)))
     (else
      (if (##wr-stamp we obj)
          (if (case (macro-writeenv-style we)
                ((pretty-print)
                 (##not (##wr-one-line-pretty-print
                         we
                         obj
                         (lambda (we obj)
                           (wr-meroon we obj)))))
                (else
                 #t))
              (wr-meroon we obj)))))))
